#varpose
# 12/30/03
# Copyright (c) 2003, David G. Drumright [ockham]

from Tkinter import *
import tkFileDialog
import poser
import os
import string

scene = poser.Scene()
BasePathUncut=poser.AppLocation()
BasePath=BasePathUncut[:(-len('poser.exe'))]
PosePath = os.path.normpath(BasePath + '/runtime/libraries/pose/*')

TheFigure = scene.CurrentFigure()
if not TheFigure: raise 'No figure selected'

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("VarPose")

        # Establish the two main dicts
        self.FigBase ={}
        self.FigPosed={}

        self.StatusEntry=Entry(self.master,width=50)
        self.StatusEntry.grid(row=0,column=1,columnspan=2)

        # Lists
        #self.ListScroll = Scrollbar(self.master, orient=VERTICAL)
        #self.ListScroll.grid( row=2, column=0,sticky=N+S+E)
        #self.List = Listbox(self.master, height=5, selectmode=SINGLE,yscrollcommand = self.ListScroll.set)
        #self.List.grid( row=2, column=1)
        # Set the scrollbar to work
        #self.ListScroll["command"] = self.List.yview

        # Slider
        self.Slider = Scale(self.master, from_=0.0, to=100.0,orient=HORIZONTAL,resolution=1.0)
        self.Slider.grid(row=6, column=0,columnspan=3)
        self.Slider.set(100.0)


        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Apply scale", command=self.HandleGo)
        self.buttonGo.grid(row=7, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=7, column=2)

        # sample binding
        #self.List.bind("<Double-Button-1>",self.HandleSelect)

        # NEW: Exit cleanly
        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

        # Get the default before anything else:
        self.HoldDefaultPose()

        # Now present the file dialog.

        fn=tkFileDialog.askopenfilename(initialfile=PosePath, filetypes=[("Poses", "pz2")] )

        if not fn: self.ShowStatus('No file selected')
        else:
            self.FileName = os.path.normpath(fn)
            self.ShowStatus(self.FileName)


    # - - - - - - - - - - - - - - - - - -
    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def HoldDefaultPose(self):
        # Pick up default settings of all usable parms on the figure.
        for OneActor in TheFigure.Actors():
            ParmList = []
            for OneParm in OneActor.Parameters():
                if not OneParm.Hidden():
                    ParmList.append( (OneParm.Name(),OneParm.Value()) )
            self.FigBase[OneActor.Name()]=ParmList

    # - - - - - - - - - - - - - - - - - -

    def HoldFullPose(self):
        # Pick up settings of all usable parms on the figure...... this
        # is to be called after applying the PZ2.
        for OneActor in TheFigure.Actors():
            ParmList = []
            for OneParm in OneActor.Parameters():
                if not OneParm.Hidden():
                    ParmList.append( (OneParm.Name(),OneParm.Value()) )
            self.FigPosed[OneActor.Name()]=ParmList

    # - - - - - - - - - - - - - - - - - -

    def ModulateFile(self, Scale):
        for ActorName in self.FigBase.keys():
            # Get the actual Actor
            TheActor = TheFigure.Actor(ActorName)
            BaseParmList = self.FigBase[ActorName]
            # Get the PosedParmList corresponding:
            PosedParmList = self.FigPosed[ActorName]
            for (OnePosedName,OnePosedValue) in PosedParmList:
                # Locate corresponding val in Base:
                for (OneBaseName,OneBaseValue) in BaseParmList:
                    if (OnePosedName == OneBaseName) and (OnePosedValue != OneBaseValue):
                        ScaledVal = (OnePosedValue - OneBaseValue) * Scale
                        TheActor.Parameter(OnePosedName).SetValue(ScaledVal)

    # - - - - - - - - - - - - - - - - - -

    def Update(self):  # Share the wealth
        if scene: scene.ProcessSomeEvents(1)
        root.lift()
        root.after(250, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleSelect(self,event): # sample handler
        # called when doubleclick in list.
        Selecteds = event.widget.curselection()
        Index = int(Selecteds[0])  #Selecteds is tuple of indexes, but this box is SINGLE

    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        # Pick up the scale value
        RawScale = float(self.Slider.get())
        Scale = 0.01 * RawScale
        self.ShowStatus("applying %d%%" % RawScale)
        # Apply the full pose
        scene.LoadLibraryPose(self.FileName)
        # Get the full values
        self.HoldFullPose()
        # Scale it down
        self.ModulateFile(Scale)
        scene.DrawAll()


    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()  # degug sharing
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
